@@ -3,7 +3,7 @@
 namespace TsLib\Rest\Services;
 
 use Illuminate\Support\Facades\Log;
-use TsLib\Rest\Models\NsRestletRequest;
+use TsLib\Models\NsRestletRequest;
 
 class Curl
 {
@@ -36,6 +36,7 @@ class Curl
     private $oauth_version = "1.0";
     private $oauth_signatureMethod = "HMAC-SHA256";
 
+    protected $orig_opts = [];
     protected $opts = [];
 
     /**
@@ -60,6 +61,7 @@ class Curl
         {
             $this->opts[$opt] = $value;
         }
+        $this->orig_opts = $this->opts;
     }
 
     /**
@@ -70,9 +72,8 @@ class Curl
      * @param boolean $assoc
      * @return mixed
      */
-    public function get($path, $params = null, $assoc = false) {
-        $this->opts[CURLOPT_CUSTOMREQUEST] = 'GET';
-        $exec = $this->execute($path, [], $params, $assoc);
+    public function get($path, $params = null, $assoc = false, $log = true) {
+        $exec = $this->execute($path, [CURLOPT_CUSTOMREQUEST => 'GET'], $params, $assoc, $log);
 
         return $exec;
     }
@@ -84,18 +85,18 @@ class Curl
      * @param array $params
      * @return mixed
      */
-    public function post($path, $body = null, $params = array(), $json = false) {
+    public function post($path, $body = null, $params = array(), $json = true, $assoc = false, $log = true) {
         $opts = [];
         if($json) {
             $body = json_encode($body);
-            $opts[CURLOPT_HTTPHEADER] = array('Content-Type: application/json');
+            $opts[CURLOPT_HTTPHEADER] = ['Content-Type: application/json'];
         }
 
         $opts[CURLOPT_POST] = true;
         $opts[CURLOPT_CUSTOMREQUEST] = 'POST';
         $opts[CURLOPT_POSTFIELDS] = $body;
 
-        $exec = $this->execute($path, $opts, $params);
+        $exec = $this->execute($path, $opts, $params, $assoc, $log);
 
         return $exec;
     }
@@ -107,14 +108,14 @@ class Curl
      * @param array $params
      * @return mixed
      */
-    public function post_multipart($path, $body = null, $params = array()) {
+    public function post_multipart($path, $body = null, $params = array(), $assoc = false, $log = true) {
         $opts = array(
             CURLOPT_HTTPHEADER => array('content-type: multipart/form-data'),
             CURLOPT_POST => true, 
             CURLOPT_POSTFIELDS => $body
         );
 
-        $exec = $this->execute($path, $opts, $params);
+        $exec = $this->execute($path, $opts, $params, $assoc, $log);
 
         return $exec;
     }
@@ -127,7 +128,7 @@ class Curl
      * @param array $params
      * @return mixed
      */
-    public function put($path, $body = null, $params, $json = false) {
+    public function put($path, $body = null, $params, $json = false, $assoc = false, $log = true) {
         if($json)
             $body = json_encode($body);
 
@@ -137,7 +138,7 @@ class Curl
             CURLOPT_POSTFIELDS => $body
         );
         
-        $exec = $this->execute($path, $opts, $params);
+        $exec = $this->execute($path, $opts, $params, $assoc, $log);
 
         return $exec;
     }
@@ -149,12 +150,12 @@ class Curl
      * @param array $params
      * @return mixed
      */
-    public function delete($path, $params) {
+    public function delete($path, $params = [], $assoc = false, $log) {
         $opts = array(
             CURLOPT_CUSTOMREQUEST => "DELETE"
         );
         
-        $exec = $this->execute($path, $opts, $params);
+        $exec = $this->execute($path, $opts, $params, $assoc, $log);
         
         return $exec;
     }
@@ -189,6 +190,16 @@ class Curl
             return $ch;
         }
 
+        if($this->auth['type'] == 'header_bearer')
+        {
+            if(!isset($this->auth['token']) || trim($this->auth['token']) == '')
+            {
+                return $ch;
+            }
+            $this->opts[CURLOPT_HTTPHEADER][] = 'Authorization: Bearer '.$this->auth['token'];
+            return $ch;
+        }
+
         if($this->auth['type'] == 'header_token')
         {
             if(!isset($this->auth['token']) || trim($this->auth['token']) == '')
@@ -199,6 +210,16 @@ class Curl
             return $ch;
         }
 
+        if($this->auth['type'] == 'header_auth')
+        {
+            if(!isset($this->auth['token']) || trim($this->auth['token']) == '')
+            {
+                return $ch;
+            }
+            $this->opts[CURLOPT_HTTPHEADER][] = 'authorization: '.$this->auth['token'];
+            return $ch;
+        }
+
         if($this->auth['type'] == 'api_token')
         {
             if(!isset($this->auth['token']) || trim($this->auth['token']) == '')
@@ -250,10 +271,6 @@ class Curl
 
         $getParams = isset($url_info['query']) ? $url_info['query'] : '';
 
-        $restlet = [];
-        if($getParams!=""){
-            $restlet = explode('&',$getParams);
-        }
         $baseString = strtoupper($httpMethod).'&'.rawurlencode($baseUrl).'&';
 
         $params = [];
@@ -263,10 +280,10 @@ class Curl
             foreach($params_array as $par)
             {
                 $separated = explode('=', $par);
-                $params[$separated[0]] = $separated[1];
+                $params[urldecode($separated[0])] = urldecode($separated[1]);
             }
         }
-       
+
         $params['oauth_consumer_key'] = config('ts-lib-rest.consumer_key');
         $params['oauth_nonce'] = $nonce;
         $params['oauth_signature_method'] = $this->oauth_signatureMethod;
@@ -275,7 +292,7 @@ class Curl
         $params['oauth_version'] = $this->oauth_version;
         ksort($params);
 
-        $baseString .= rawurlencode(http_build_query($params));
+        $baseString .= rawurlencode(http_build_query($params, '', '&', PHP_QUERY_RFC3986));
 
         $key = rawurlencode(config('ts-lib-rest.consumer_secret')) .'&'. rawurlencode(config('ts-lib-rest.token_secret'));
 
@@ -311,20 +328,45 @@ class Curl
 
         // array_merge destruye los indices del array, por eso tengo que hacer esto para unificarlos al instanciar
         // tambien aqui le doy chance de agregar mas opciones al llamar la funcion, como cuando se agrega CURLOPT_CUSTOMREQUEST
+        $this->opts = $this->orig_opts;
         foreach($opts as $opt => $value)
         {
-            $this->opts[$opt] = $value;
+            if(isset($this->orig_opts[$opt]) && is_array($this->orig_opts[$opt]))
+            {
+                $this->opts[$opt] = array_merge($this->orig_opts[$opt], $value);
+            }
+            else
+            {
+                $this->opts[$opt] = $value;
+            }
         }
 
         $log_request = new NsRestletRequest();
-        $log_request->url = $uri;
+        $log_request->url = $this->API_ROOT_URL;
+        $log_request->path = $path;
+        $log_request->query = json_encode($params);
+
+        $url_info = parse_url($uri);
+        if(isset($url_info['query']))
+        {
+            $out = [];
+            parse_str($url_info['query'], $out);
+            if(isset($out['script']))
+            {
+                $log_request->script_id = $out['script'];
+            }
+        }
+        
         $log_request->method = $this->opts[CURLOPT_CUSTOMREQUEST];
-        if($log)
-            $log_request->json = isset($this->opts[CURLOPT_POSTFIELDS]) ? $this->opts[CURLOPT_POSTFIELDS] : '';
-        else
-            $log_request->json = '';
         $log_request->status = 0;
-        $log_request->save();
+        if(isset($this->opts[CURLOPT_POSTFIELDS]))
+        {
+            $log_request->json = $this->opts[CURLOPT_POSTFIELDS];
+        }
+        else
+        {
+            $log_request->json = '';
+        }
 
         $this->opts[CURLOPT_URL] = $uri;
 
@@ -335,6 +377,9 @@ class Curl
         curl_setopt_array($ch, $this->opts);
 
         $data = curl_exec($ch);
+        $response_info = curl_getinfo($ch);
+        $req_headers = explode(PHP_EOL, $response_info['request_header']);
+        $return['request_header'] = $req_headers;
 
         if($data === false)
         {
@@ -342,18 +387,28 @@ class Curl
             $log_request->status = 1;
             $log_request->save();
             curl_close($ch);
-            return ["status" => false, "httpCode" => 500, "data" => [], "message"=>"Error curl", 'headers' => []];
+            return ["status" => false, "httpCode" => 500, "body" => [], "message"=>"Error curl", 'headers' => [], 'request_header' => $req_headers, 'uri' => $uri];
         }
 
-        $response_content_type = explode(';', curl_getinfo($ch, CURLINFO_CONTENT_TYPE));
+        if(($response_content_type_header = curl_getinfo($ch, CURLINFO_CONTENT_TYPE)) != false)
+        {
+            $response_content_type = explode(';', $response_content_type_header);
+            $parts_rct = explode('/', $response_content_type[0]);
+            $response_content_type[0] = explode('+', $parts_rct[1]);
+        }
+        else
+        {
+            $response_content_type[0] = [];
+        }
 
         $headerSize = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
         $return["httpCode"] = curl_getinfo($ch, CURLINFO_HTTP_CODE);
         curl_close($ch);
+
         $header = substr($data, 0, $headerSize);
         $body = substr($data, $headerSize);
 
-        if($response_content_type[0] == 'application/json')
+        if($response_content_type[0] == 'json' || in_array('json', $response_content_type[0]))
             $return["body"] = json_decode($body, $assoc);
         else
             $return["body"] = $body;
@@ -373,18 +428,29 @@ class Curl
             }
         }
         $return['headers'] = $responseHeaders;
+        $return['uri'] = $uri;
 
-        $log_request->response = $return["body"];
-        if(in_array($result["httpCode"], [200, 201, 202, 203]) &&
-            $return["body"] !== false &&
-            isset($return["body"]->code) &&
-            $return["body"]->code == 1)
-            {
-                $log_request->status = 9;
-            }
+        $log_request->response = json_encode($return);
+        if(!in_array($return["httpCode"], [200, 201, 202, 203]))
+        {
+            $log_request->status = 2;
+            $log_request->save();
+            return $return;
         }
-        $log_request->save();
         
+        if($return["body"] === false)
+        {
+            $log_request->status = 1;
+            $log_request->save();
+            return $return;
+        }
+        
+        $log_request->status = 9;
+        if($log)
+        {
+            $log_request->save();
+        }
+
         return $return;
     }
 
@@ -406,15 +472,9 @@ class Curl
         }
 
         if(!empty($params)) {
-            $paramsJoined = array();
-
-            foreach($params as $param => $value) {
-               $paramsJoined[] = "$param=$value";
-            }
-            $params = '?'.implode('&', $paramsJoined);
-            $uri = $uri.$params;
+            $uri .= '?'.http_build_query($params, '', null, PHP_QUERY_RFC3986);
         }
 
         return $uri;
     }
-}
+}
\ No newline at end of file
