@@ -15,27 +15,22 @@ class NetsuiteBase {
         date_default_timezone_set(config('app.timezone', 'America/Mexico_City'));
     }
 
-    public function restlet($httpMethod, $name, $params, $json='')
+    public function get($path, $params = [], $log = true)
     {
-        Log::error('Llamando NetsuiteBase->restlet sin extenderlo');
-        return false;
+        return $this->curl->get($path, $params, $log);
     }
 
-    public function rest($httpMethod, $path, $json = '')
+    public function post($path, $json, $log = true)
     {
-        Log::error('Llamando NetsuiteBase->rest sin extenderlo');
-        return false;
+        return $this->curl->post($path, $json, $log);
     }
 
     protected function doRequest($httpMethod,$path,$json="", $log = true)
     {
-        $resp = ["status" => false,"data" => [],"message"=>"Error api get"];
-
         if($httpMethod == "POST"){
             $validateJSON = $this->_jsonValidator($json);
             if($validateJSON != 1){
-                $response["message"] = "JSON error";
-                return $resp;
+                return ["status" => false, "httpCode" => 0, "data" => [], "message"=>"Error NetSuiteBase: No body", 'headers' => []];
             }
         }
 
@@ -53,45 +48,7 @@ class NetsuiteBase {
         }
 
         //hacer CURL
-        $result = $this->curl->execute($path, $options, [], false, $log);
-
-        if(!$result)
-        {
-            return $resp;
-        }
-
-        if(!in_array($result["httpCode"], [200, 201, 202, 203]))
-        {
-            $resp["data"] = [];
-            $resp['message'] =  isset($result['body']->error->message) ? $result['body']->error->message : "No se recibio un resultado exitoso";
-            $resp["status"] = false;
-
-            return $resp;
-        }
-
-        if($result['body']->code)
-        {
-            if(isset($result['body']->data))
-            {
-                $resp["data"] = $result['body']->data;
-            }
-            else
-            {
-                $resp["data"] = [];
-            }
-            $resp['message'] = $result['body']->message;
-            $resp["status"] = true;
-        }
-        else
-        {
-            $resp["status"] = false;
-            if(isset($result["body"]->message))
-            {
-                $resp["message"] = $result["body"]->message;
-            }
-        }
-
-        return $resp;
+        return $this->curl->execute($path, $options, [], false, $log);
     }
 
     public function _jsonValidator($json)
